#!/bin/bash
# test_sparse.sh [PLUGINS] [ENCODE] [DECODE]
#
# Tests whether we deal with sparsity correctly
#

TESTFILE=testfile

do_copy()
# $1: "-a" (or "-a -b 16k") or empty
# $2: plugin: "-L ./libddr_xxx=xxx"
# $3: encrypt or compress ...
# $4: decrypt or decompress ...
{
	rm -f ${TESTFILE}.copy2 ${TESTFILE}.copy
	if test -z "$3"; then
		echo dd_rescue $1 $2 ${TESTFILE} ${TESTFILE}.copy || return
		./dd_rescue $1 $2 ${TESTFILE} ${TESTFILE}.copy || return
	elif test -z "$4"; then
		echo dd_rescue $1 $2$3 ${TESTFILE} ${TESTFILE}.copy || return
		./dd_rescue $1 $2$3 ${TESTFILE} ${TESTFILE}.copy || return
	else
		echo dd_rescue $1 $2$3 ${TESTFILE} ${TESTFILE}.copy2 || return
		./dd_rescue $1 $2$3 ${TESTFILE} ${TESTFILE}.copy2 || return
		echo dd_rescue $1 $2$4 ${TESTFILE}.copy2 ${TESTFILE}.copy || return
		./dd_rescue $1 $2$4 ${TESTFILE}.copy2 ${TESTFILE}.copy || return
		rm ${TESTFILE}.copy2
	fi
}

my_exit()
{
	echo "ERROR $1: $2" 1>&2
	rm -f ${TESTFILE}.copy2 ${TESTFILE}.copy ${TESTFILE}
	exit $1
}

# Create sparse file
echo "./dd_rescue -Z 0 -m 16M ${TESTFILE}
./dd_rescue -S 32k -m 16k /dev/zero ${TESTFILE}
./dd_rescue -S 1M -m 1M /dev/zero ${TESTFILE}
./dd_rescue -S 3M -m 1M /dev/zero ${TESTFILE}
"
./dd_rescue -Z 0 -m 16M ${TESTFILE}
./dd_rescue -S 32k -m 16k /dev/zero ${TESTFILE}
./dd_rescue -S 1M -m 1M /dev/zero ${TESTFILE}
./dd_rescue -S 3M -m 1M /dev/zero ${TESTFILE}
# Without -a
do_copy "" "$1" "$2" "$3"
ERR=$?
if test $ERR != 0; then my_exit $ERR "Copy error"; fi
cmp ${TESTFILE} ${TESTFILE}.copy
ERR=$?
if test $ERR != 0; then my_exit $((ERR+32)) "Compare error"; fi
# With -a
do_copy "-a -b 16k" "$1" "$2" "$3"
ERR=$?
if test $ERR == 13; then
	echo "INFO: sparse not supported with "${1%%=*}" (no error)"
elif test $ERR != 0; then
	my_exit $((ERR+64)) "Error with sparse"
else
	du ${TESTFILE} ${TESTFILE}.copy
	cmp ${TESTFILE} ${TESTFILE}.copy
	ERR=$?
	if test $ERR != 0; then my_exit $((ERR+96)) "Sparse compare error"; fi
fi
rm -f ${TESTFILE}.copy2 ${TESTFILE}.copy ${TESTFILE}
